# Copyright (C) 2024 ETH Zurich, Institute for Particle Physics and Astrophysics

import hashlib
import os
import shutil
import sys
import tempfile
import zipfile
from urllib.request import urlretrieve

BASE_URL = os.environ.get("COSMO_TORRENT_BASE_URL", "https://share.phys.ethz.ch/~ipa/")
DATA_FOLDER = os.environ.get(
    "COSMO_TORRENT_DATA_FOLDER",
    "login.phys.ethz.ch:/net/ipa-gate.phys.ethz.ch/export/ipa/public_html/",
)

MARKER_FILE = ".valid"


def data_path(identifier):
    cache_folder = local_cache_folder(identifier)
    if not is_valid(cache_folder):
        download_data(identifier, cache_folder)
    return cache_folder


def prepare_upload(folder, identifier):
    prep_folder = tempfile.mkdtemp()
    shutil.make_archive(os.path.join(prep_folder, identifier), "zip", ".", folder)
    md5_sum = _md5_sum(os.path.join(prep_folder, identifier) + ".zip")
    with open(os.path.join(prep_folder, identifier) + ".md5", "w") as fh:
        fh.write(md5_sum)
    os.system(f"scp {prep_folder}/* schmittu@{DATA_FOLDER}")


def _md5_sum(path, CHUNK_SIZE=512 * 1024**2):
    digest = hashlib.md5()
    with open(path, "rb") as fh:
        while True:
            chunk = fh.read(CHUNK_SIZE)
            if not chunk:
                return digest.hexdigest()
            digest.update(chunk)


def local_cache_folder(identifier):
    return os.path.join(_cache_base_folder(), identifier)


def _cache_base_folder():
    home = os.path.expanduser("~")
    if sys.platform.startswith("darwin"):
        base_folder = os.path.join(home, "Library", "Cache")
    else:
        base_folder = os.path.join(home, "_cache")
    return base_folder


def is_valid(cache_folder):
    return os.path.exists(os.path.join(cache_folder, MARKER_FILE))


def download_data(identifier, cache_folder):
    archive = _download_and_check_data(identifier, cache_folder)

    with zipfile.ZipFile(archive, "r") as zh:
        zh.extractall(cache_folder)


def _download_and_check_data(identifier, cache_folder):
    for _ in range(3):
        with tempfile.TemporaryDirectory() as download_folder:
            archive = identifier + ".zip"
            downloaded_archive = os.path.join(download_folder, archive)
            url = BASE_URL.rstrip("/") + "/" + archive
            urlretrieve(url, downloaded_archive)

            md5 = identifier + ".md5"
            downloaded_md5 = os.path.join(download_folder, md5)
            url = BASE_URL.rstrip("/") + "/" + md5
            urlretrieve(url, downloaded_md5)

            if check_downloaded_data(downloaded_archive, downloaded_md5):
                return downloaded_archive
    else:
        raise IOError(
            "was not able to download data after 3 attempts without checksum issues"
        )


def check_downloaded_data(downloaded_archive, downloaded_md5):
    with open(downloaded_md5, "r") as fh:
        md5sum = fh.read().strip()

    with open(downloaded_archive, "rb") as fh:
        while True:
            chunk = fh.read(1_000_000)
            if not chunk:
                break
            digest.update(chunk)

    return digest.hexdigest() == md5


def _download_data_archive(identifier, cache_folder):
    url = BASE_URL.rstrip("/") + "/" + identifier + ".zip"
    urlretrieve(url, cache_folder)


if __name__ == "__main__":
    prepare_upload(".", "ucat.test")
    data_path("ucat.test")
